/* * * * * * * * * * * * * * * * * * * * * * * *
 * SCIDYNE Corporation, Oct 12, 2021,  Mark Durgin 
 * Simple digital I/O demo program for MULTI-IO-ARD PN #100-7714
 * - Repeatedly writes pattern of alternating 0's and 1's to Port-A of the MCP23017 chip
 *   Use an oscilloscope to observe results
 * - Reads state of Port-B and prints to serial monitor. Short PortB channel pins to ground to cause changes
 */
#include <Wire.h>   // Needed for Arduino I2C support.

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Program constants and defines
 */
#define COMPORT Serial                  // Serial Port to use: UNO, MEGA use Serial,  DUE may use Serial1
#define WIREPORT Wire                   // Set Wire Library to use: UNO, MEGA use Wire, DUE uses Wire1

// Define the MCP23017 address and registers to be used. See the MCP23017 data sheet for details
#define MCP23017_ADDRESS      0x20      // Chip Address A0 = A1 = A3 = GND.
#define MCP23017_IODIRA       0x00      // MCP23017 Port-A Data Direction Register
#define MCP23017_GPIOA        0x12      // MCP23017 Port-A Data Register
#define MCP23017_IODIRB       0x01      // MCP23017 Port-B Data Direction Register
#define MCP23017_GPIOB        0x13      // MCP23017 Port-B Data Register
#define MCP23017_GPPUB        0x0D      // MCP23017 Port-B Pull-Up enable Register

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * setup runs once then execution continues at loop()
 */
void setup()
{
  WIREPORT.begin();                           // Setup I2C communications

  COMPORT.begin(9600);                    // Setup communications to IDE Monitor
    while (!COMPORT);

  // Setup MCP23017 Port-A as all outputs
  WIREPORT.beginTransmission(MCP23017_ADDRESS);       // Open I2C communication with the MCP23017
  WIREPORT.write(MCP23017_IODIRA);                    // Next byte intended for Port-A direction register
  WIREPORT.write(0x00);                               // Make all outputs, Bits set as 0 are outputs
  WIREPORT.endTransmission();                         // Close I2C communications

  // Setup MCP23017 Port-B as all inputs
  WIREPORT.beginTransmission(MCP23017_ADDRESS);       // Open I2C communication with the MCP23017
  WIREPORT.write(MCP23017_IODIRB);                    // Next byte intended for Port-B direction register
  WIREPORT.write(0xFF);                               // Make all inputs, Bits set as 1 are inputs
  WIREPORT.endTransmission();                         // Close I2C communications

  WIREPORT.beginTransmission(MCP23017_ADDRESS);       // Open I2C communication with the MCP23017
  WIREPORT.write(MCP23017_GPPUB);                     // Next byte intended for Port-B Pull-Up register
  WIREPORT.write(0xFF);                               // Bits set as 1 enable corresponding pull-up
  WIREPORT.endTransmission();                         // Close I2C communications

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * loop runs continuously
 */
void loop()
{
  // Toggle bit pattern on Port-A
  WIREPORT.beginTransmission(MCP23017_ADDRESS);      // Open I2C communication with the MCP23017
  WIREPORT.write(MCP23017_GPIOA);                    // Next byte intended for Port-A data register
  WIREPORT.write(0xAA);                              // Make Port-A = 1 0 1 0 1 0 1 0
  WIREPORT.endTransmission();                        // Close I2C communications

  //delay(100);                                  // Wait 100mS
  
  WIREPORT.beginTransmission(MCP23017_ADDRESS);      // Open I2C communication with the MCP23017
  WIREPORT.write(MCP23017_GPIOA);                    // Next byte intended for Port-A data register
  WIREPORT.write(0x55);                              // Make Port-A = 0 1 0 1 0 1 0 1
  WIREPORT.endTransmission();                        // Close I2C communications

  //delay(100);                                  // Wait 100mS

  // Read the the state of Port-B and display on Arduino IDE Serial Monitor
  WIREPORT.beginTransmission(MCP23017_ADDRESS);      // Open I2C communication with the MCP23017
  WIREPORT.write(MCP23017_GPIOB);                    // Read will be from Port-B data register
  WIREPORT.endTransmission();

  WIREPORT.requestFrom(MCP23017_ADDRESS, 1);         // Request 1 byte from the MCP23017
  unsigned char x = WIREPORT.read();                 // Receive the Port-B byte and store in x
  COMPORT.println(x);                             // Print the character
  
  //delay(100);                                  // Wait 100mS

}
